#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/thread.hpp>
#include <iostream>
#include <list>

typedef struct _Param
{
  int callCnt;
  int loopCnt;
  int threadCnt;
} Param;

void foo(int loopCnt)
{
  int cnt = loopCnt;
  for (int i = 0; i < cnt; i++);
}

void threadProc(Param* param)
{
  int callCnt = param->callCnt;
  int loopCnt = param->loopCnt;

  boost::system_time::ptime begTick(boost::posix_time::microsec_clock::local_time());
  
  for (int i = 0; i < callCnt; i++)
  {
    foo(loopCnt);
  }
  
  boost::system_time::ptime endTick(boost::posix_time::microsec_clock::local_time());

  std::string s = boost::posix_time::to_simple_string(endTick - begTick) += "\n"; std::cout << s;
}

void usage()
{
  std::cout << "call_loop_thread_test <call count> <loop count> <thread count>" << std::endl;
  std::cout << "example : call_loop_thread_test 100 100 2" << std::endl;
}

int main(int argc, char* argv[])
{
  if (argc != 4)
  {
    usage();
    return 0;
  }

  Param param;

  param.callCnt   = boost::lexical_cast<int>(argv[1]);
  param.loopCnt   = boost::lexical_cast<int>(argv[2]);
  param.threadCnt = boost::lexical_cast<int>(argv[3]);

  std::list<boost::thread*> threadList;

  for (int i = 0; i < param.threadCnt; i++)
  {
    boost::thread* t = new boost::thread(threadProc, &param);
    threadList.push_back(t);
  }

  for (std::list<boost::thread*>::iterator it = threadList.begin(); it != threadList.end(); it++)
  {
    boost::thread* t = *it;
    t->join();
  }

  threadList.clear();
  return 0;
}
